package ru.dbotthepony.mc.otm.data.world

import com.mojang.serialization.Codec
import com.mojang.serialization.codecs.RecordCodecBuilder
import net.minecraft.util.RandomSource
import net.minecraft.util.valueproviders.FloatProvider
import net.minecraft.util.valueproviders.UniformFloat

class BooleanProvider(val sampler: FloatProvider, val threshold: Float, val lessThan: Boolean = true) {
	fun sample(random: RandomSource): Boolean {
		val sampled = sampler.sample(random)

		if (lessThan) {
			return sampled < threshold
		} else {
			return sampled >= threshold
		}
	}

	companion object {
		fun onceEvery(samples: Int): BooleanProvider {
			return BooleanProvider(UniformFloat.of(0f, samples.toFloat()), 1f, lessThan = true)
		}

		val CODEC: Codec<BooleanProvider> = RecordCodecBuilder.create {
			it.group(
				FloatProvider.CODEC.fieldOf("sampler").forGetter(BooleanProvider::sampler),
				Codec.FLOAT.fieldOf("threshold").forGetter(BooleanProvider::threshold),
				Codec.BOOL.optionalFieldOf("less_than", true).forGetter(BooleanProvider::lessThan)
			).apply(it, ::BooleanProvider)
		}
	}
}
